<?php

namespace Drupal\entityreference_extensions\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\Plugin\Field\FieldFormatter\EntityReferenceEntityFormatter;
use Drupal\Core\Form\FormStateInterface;
use Drupal\entityreference_extensions\EntityReferenceDeltaFilterTrait;

/**
 * Plugin implementation of the 'entity reference rendered entity delta' formatter.
 *
 * @FieldFormatter(
 *   id = "entity_reference_entity_view_delta",
 *   label = @Translation("Rendered entity (delta)"),
 *   description = @Translation("Display the referenced entities rendered by entity_view(). Limit multi value field by delta."),
 *   field_types = {
 *     "entity_reference"
 *   }
 * )
 */
class EntityReferenceEntityDeltaFormatter extends EntityReferenceEntityFormatter {

  use EntityReferenceDeltaFilterTrait;

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
        'delta' => 'all',
      ] + parent::defaultSettings();
  }
  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $elements = parent::settingsForm($form, $form_state);
    if ($this->field_cardinality_valid()) {
      $options = [
        'all' => $this->t('All'),
      ];
      for ($i = 0; $i < 10; $i++) {
        $options[$i] = $i + 1;
      }
      $elements['delta'] = [
        '#type' => 'select',
        '#title' => $this->t('Limit elements'),
        '#options' => $options,
        '#default_value' => $this->getSetting('delta'),
      ];
    }
    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = parent::settingsSummary();
    if ($this->field_cardinality_valid()) {
      $summary[] = $this->getSetting('delta') == 'all' ? t('Show all') : t('Show @item', ['@item' => $this->getSetting('delta') + 1]);
    }
    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    if($this->getSetting('delta') != 'all' && $this->field_cardinality_valid()) {
      $items->filter([$this, 'delta_filter']);
    }
    $elements = parent::viewElements($items, $langcode);
    return $elements;
  }


}
